/*
 * gtkdlg.c - GTK implementation of the PuTTY configuration box.
 */

/*
 * TODO when porting to GTK 2.0:
 * 
 *  - GtkTree is apparently deprecated and we should switch to
 *    GtkTreeView instead.
 *  - GtkLabel has a built-in mnemonic scheme, so we should at
 *    least consider switching to that from the current adhockery.
 */

#include <basic.h>
#include <bassert.h>
#include <stdarg.h>
#include <bctype.h>
#include <btron/hmi.h>
#include <btron/dp.h>
#include <btron/libapp.h>

#include "putty.h"
#include "tcsliteral.h"

#define MAXITEMS  (1+20)
#define MAXWIDTH  35
#define GAPH      24
#define GAPV      24
#define GAPSW     8

void error_panel(const TC *p)
{
  static TC buf0[MAXWIDTH*MAXITEMS+1];
  static TC ms0txt[] = {TCS_BTN_CONFIRM};
  static SWSEL	ms0 = {
    MS_PARTS|P_DISP|P_EMPHAS,
    {{0, 0, 80, 24}},
    0,
    ms0txt
  };
	
  static PNL_ITEM pi0[MAXITEMS] = {
    {PARTS_ITEM, 0, {{0, 0, 0, 0}}, 0, 0, (H*)&ms0}
  };
  
  static PANEL pn0 = {
    4, 0x784, 0, {{0, 0, 0, 0}}, 1, 1, pi0
  };
	
  WEVENT wev1;
  PNT p0;
  TC *q;
  W maxw, y, i, pid0;
	
  maxw = 10;
  q = buf0;
 
  for (y=1; ((y < MAXITEMS)&&(p != NULL)); y++) {
    pi0[y].itype = TEXT_ITEM;
    pi0[y].info = 0;
    pi0[y].desc = 0;
    pi0[y].dnum = 0;
    pi0[y].ptr = (H*)q;
		
    i = 0;
    for (;;) {
      switch (*(q++) = *(p++)) {
          case	TNULL:
            p = NULL;
            break;
          case	TK_NL:
            q[-1] = TNULL;
            break;
          default:
            if ((++i) < MAXWIDTH)
              continue;
            *(q++) = TNULL;
            break;
      }
      break;
    }
    
    pi0[y].ir.c.left = GAPH;
    pi0[y].ir.c.top = GAPV + 20 * (y - 1);
    pi0[y].ir.c.right = GAPH + 16 * i;
    pi0[y].ir.c.bottom = GAPV + 20 * (y - 1) + 16;
		
    if (maxw < i)
      maxw = i;
  }
	
  pn0.nitem = y;
  pn0.r.c.right = GAPH * 2 + 16 * maxw;
  pn0.r.c.bottom = GAPV * 2 + 20 * (y - 1) + GAPSW + ms0.r.c.bottom;
  pi0[0].ir.c.left = GAPH + (16 * maxw - ms0.r.c.right) / 2;
  pi0[0].ir.c.top = GAPV + 20 * (y - 1) + GAPSW;
	
  p0.x = (SCREEN.hpixels - pn0.r.c.right) / 2;
  p0.y = (SCREEN.vpixels - pn0.r.c.bottom) / 2;
	
  pid0 = pcre_pnl(&pn0, &p0);
  
  sig_buz(0);
  if (pid0 < 0) {
    pdsp_msg(buf0);
    return;
  }
  for (;;) {
    switch (pact_pnl(pid0, &wev1.e, &i)) {
        case	P_EVENT:
          if (wev1.s.type == EV_DEVICE)
            oprc_dev(&wev1.e, NULL, 0);
          continue;
        case	0x5001:
          break;
        default:
          if (i >= 0)
            continue;
    }
    break;
  }
 
  pdel_pnl(pid0);
  return;
}

void modalfatalbox(char *p)
{
  TC s[310];
  eucstotcs(s, p);
  error_panel(s);
}

void fatal_message_box(void *window, char *msg)
{
  TC *msg_tc;

  msg_tc = snewn(strlen(msg) + 1, TC);
  eucstotcs(msg_tc, msg);
  error_panel(msg_tc);
  sfree(msg_tc);
}

void fatalbox(char *p, ...)
{
  va_list ap;
  char *msg;
  va_start(ap, p);
  msg = dupvprintf(p, ap);
  va_end(ap);
  fatal_message_box(NULL, msg);
  sfree(msg);
  cleanup_exit(1);
}

int confirm_panel(const TC *p)
{
  static TC buf0[MAXWIDTH*MAXITEMS+1];
  static TC ms0txt[] = {TCS_BTN_CANCEL};
  static TC ms1txt[] = {TCS_BTN_CONTINUE};
  static SWSEL	ms0 = {
    MS_PARTS|P_DISP,
    {{0, 0, 80, 24}},
    0,
    ms0txt
  };
  static SWSEL	ms1 = {
    MS_PARTS|P_DISP|P_EMPHAS,
    {{0, 0, 80, 24}},
    0,
    ms1txt
  };
	
  static PNL_ITEM pi0[MAXITEMS] = {
    {PARTS_ITEM, 0, {{0, 0, 0, 0}}, 0, 0, (H*)&ms0},
    {PARTS_ITEM, 0, {{0, 0, 0, 0}}, 0, 0, (H*)&ms1},
  };
  
  static PANEL pn0 = {
    4, 0x784, 0, {{0, 0, 0, 0}}, 2 /* defsw */, 1, pi0
  };
	
  WEVENT wev1;
  PNT p0;
  TC *q;
  W maxw, y, i, pid0;
  int ret;
	
  maxw = 10;
  q = buf0;
 
  for (y=2; ((y < MAXITEMS)&&(p != NULL)); y++) {
    pi0[y].itype = TEXT_ITEM;
    pi0[y].info = 0;
    pi0[y].desc = 0;
    pi0[y].dnum = 0;
    pi0[y].ptr = (H*)q;
		
    i = 0;
    for (;;) {
      switch (*(q++) = *(p++)) {
          case	TNULL:
            p = NULL;
            break;
          case	TK_NL:
            q[-1] = TNULL;
            break;
          default:
            if ((++i) < MAXWIDTH)
              continue;
            *(q++) = TNULL;
            break;
      }
      break;
    }
    
    pi0[y].ir.c.left = GAPH;
    pi0[y].ir.c.top = GAPV + 20 * (y - 2);
    pi0[y].ir.c.right = GAPH + 16 * i;
    pi0[y].ir.c.bottom = GAPV + 20 * (y - 2) + 16;
		
    if (maxw < i)
      maxw = i;
  }
	
  pn0.nitem = y;
  pn0.r.c.right = GAPH * 2 + 16 * maxw;
  pn0.r.c.bottom = GAPV * 2 + 20 * (y - 2) + GAPSW + ms0.r.c.bottom;

  pi0[0].ir.c.left = GAPH;
  pi0[1].ir.c.left = GAPH + 16 * maxw - ms0.r.c.right;
  pi0[0].ir.c.top = pi0[1].ir.c.top = GAPV + 20 * (y - 2) + GAPSW;	
  p0.x = (SCREEN.hpixels - pn0.r.c.right) / 2;
  p0.y = (SCREEN.vpixels - pn0.r.c.bottom) / 2;
	
  pid0 = pcre_pnl(&pn0, &p0);
  
  sig_buz(0);
  if (pid0 < 0) {
    pdsp_msg(buf0);
    return;
  }
  for (;;) {
    switch (pact_pnl(pid0, &wev1.e, &i)) {
        case	P_EVENT:
          if (wev1.s.type == EV_DEVICE)
            oprc_dev(&wev1.e, NULL, 0);
          continue;
        case	0x5001:
          break;
        default:
          if (i >= 0)
            continue;
    }
    break;
  }

  ret = (i == 2);
 
  pdel_pnl(pid0);
  return ret;
}

/*
 * Ask whether the selected algorithm is acceptable (since it was
 * below the configured 'warn' threshold).
 */
int askalg(void *frontend, const char *algtype, const char *algname,
	   void (*callback)(void *ctx, int result), void *ctx)
{
    static const char msg[] =
	"³褬󶡤%s르ꥺϡ⤢餫ꤵ줿"
	"٥򲼲äƤޤ\n"
	"%s르ꥺǤ³³ޤ\n";
    char *text;
    int ret;
    TC *text_tc;

    text = dupprintf(msg, algtype, algname);
    text_tc = snewn(strlen(text)+1, TC);
    eucstotcs(text_tc, text);
    
    ret = confirm_panel(text_tc);

    sfree(text_tc);
    sfree(text);

    if (ret) {
	return 1;
    } else {
	return 0;
    }
}

int never_once_forever_panel(const TC *p)
{
  static TC buf0[MAXWIDTH*MAXITEMS+1];
  static TC ms0txt[] = {TCS_BTN_CANCEL};
  static TC ms1txt[] = {TCS_BTN_ONCE};
  static TC ms2txt[] = {TCS_BTN_ALWAYS};
  static SWSEL	ms0 = {
    MS_PARTS|P_DISP|P_EMPHAS,
    {{0, 0, 120, 24}},
    0,
    ms0txt
  };
  static SWSEL	ms1 = {
    MS_PARTS|P_DISP,
    {{0, 0, 120, 24}},
    0,
    ms1txt
  };
  static SWSEL	ms2 = {
    MS_PARTS|P_DISP,
    {{0, 0, 120, 24}},
    0,
    ms2txt
  };
	
  static PNL_ITEM pi0[MAXITEMS] = {
    {PARTS_ITEM, 0, {{0, 0, 0, 0}}, 0, 0, (H*)&ms0},
    {PARTS_ITEM, 0, {{0, 0, 0, 0}}, 0, 0, (H*)&ms1},
    {PARTS_ITEM, 0, {{0, 0, 0, 0}}, 0, 0, (H*)&ms2},
  };
  
  static PANEL pn0 = {
    4, 0x784, 0, {{0, 0, 0, 0}}, 1 /* defsw */, 1, pi0
  };
	
  WEVENT wev1;
  PNT p0;
  TC *q;
  W maxw, y, i, pid0;
  int ret;
	
  maxw = 10;
  q = buf0;
 
  for (y=3; ((y < MAXITEMS)&&(p != NULL)); y++) {
    pi0[y].itype = TEXT_ITEM;
    pi0[y].info = 0;
    pi0[y].desc = 0;
    pi0[y].dnum = 0;
    pi0[y].ptr = (H*)q;
		
    i = 0;
    for (;;) {
      switch (*(q++) = *(p++)) {
          case	TNULL:
            p = NULL;
            break;
          case	TK_NL:
            q[-1] = TNULL;
            break;
          default:
            if ((++i) < MAXWIDTH)
              continue;
            *(q++) = TNULL;
            break;
      }
      break;
    }
    
    pi0[y].ir.c.left = GAPH;
    pi0[y].ir.c.top = GAPV + 20 * (y - 3);
    pi0[y].ir.c.right = GAPH + 16 * i;
    pi0[y].ir.c.bottom = GAPV + 20 * (y - 3) + 16;
		
    if (maxw < i)
      maxw = i;
  }
	
  pn0.nitem = y;
  pn0.r.c.right = GAPH * 2 + 16 * maxw;
  pn0.r.c.bottom = GAPV * 2 + 20 * (y - 3) + GAPSW + ms0.r.c.bottom;

  pi0[0].ir.c.left = GAPH;
  pi0[1].ir.c.left = GAPH + (16 * maxw - ms1.r.c.right) / 2;
  pi0[2].ir.c.left = GAPH + 16 * maxw - ms2.r.c.right;
  pi0[0].ir.c.top = pi0[1].ir.c.top = pi0[2].ir.c.top = GAPV + 20 * (y - 3) + GAPSW;	
  p0.x = (SCREEN.hpixels - pn0.r.c.right) / 2;
  p0.y = (SCREEN.vpixels - pn0.r.c.bottom) / 2;
	
  pid0 = pcre_pnl(&pn0, &p0);
  
  sig_buz(0);
  if (pid0 < 0) {
    pdsp_msg(buf0);
    return;
  }
  for (;;) {
    switch (pact_pnl(pid0, &wev1.e, &i)) {
        case	P_EVENT:
          if (wev1.s.type == EV_DEVICE)
            oprc_dev(&wev1.e, NULL, 0);
          continue;
        case	0x5001:
          break;
        default:
          if (i >= 0)
            continue;
    }
    break;
  }

  ret = i - 1;

  pdel_pnl(pid0);
  return ret;
}

int verify_ssh_host_key(void *frontend, char *host, int port, char *keytype,
                        char *keystr, char *fingerprint,
                        void (*callback)(void *ctx, int result), void *ctx)
{
  TC text_tc[1024];
  static const char absenttxt[] =
    "³θå¸ߤʤᡢ³ΥۥȤ"
    "ʪǤ뤳ȤݾڤǤޤ\n"
    "Ф%sŰϰʲ̤Ǥ:\n"
    "%s\n"
    "ΥۥȤꤹϡ־˵ġפ򤷤ơ"
    "å¸³³Ƥ\n"
    "⤷򥭥å¸˺¤³ʤ"
    "ֺΤ³פ򤷤Ƥ\n"
    "ޤ³򥭥󥻥뤹ˤϡǡפ򤷡"
    "³ߤƤ";
  static const char wrongtxt[] =
    "ٹ  ƥΤ\n"
    "³θå¸ƤΤȥޥåޤ"
    "ǽȤƤϡдԤۥȤθؤ뤤"
    "³褬ʪˤʤꤹޤƤ뤪줬ޤ"
    "%sŰϰʲ̤Ǥ:\n"
    "%s\n"
    "ѹꤵΤǤäʤС־˵ġפ򤷤"
    "å򹹿³³Ƥ\n"
    "⤷å򹹿˺¤³ʤ"
    "ֺΤ³פ򤷤Ƥ\n"
    "ޤ³򥭥󥻥뤹ˤϡǡפ򤷡"
    "³ߤƤʡǡװʳ򤷤硢"
    "ݾڤϤޤ";
  char *text;
  int ret;

  /*
   * Verify the key.
   */
  ret = verify_host_key(host, port, keytype, keystr);

  if (ret == 0)		       /* success - key matched OK */
    return 1;

  text = dupprintf((ret == 2 ? wrongtxt : absenttxt), keytype, fingerprint);
  eucstotcs(text_tc, text);

  ret = never_once_forever_panel(text_tc);

  sfree(text);

  if (ret == 2) {
    store_host_key(host, port, keytype, keystr);
    return 1;		       /* continue with connection */
  } else if (ret == 1)
    return 1;		       /* continue with connection */
  return 0;			       /* do not continue with connection */
}

void old_keyfile_warning(void)
{
  /*
   * This should never happen on Unix. We hope.
   */
}

