/*
 * Portable implementation of ltime() for any ISO-C platform where
 * time_t behaves. (In practice, we've found that platforms such as
 * Windows and Mac have needed their own specialised implementations.)
 */

#include <basic.h>
#include <btron/clk.h>
#include <time.h>
#include <bassert.h>

/* BTRON */
struct tm ltime(void)
{
  W err1, err2;
  STIME time;
  DATE_TIM dt;
  struct tm tm;

  err1 = get_tim(&time, NULL);
  assert(err1 >= 0);

  err2 = get_tod(&dt, time, 1);
  assert(err2 >= 0);

  tm.tm_sec = dt.d_sec;
  tm.tm_min = dt.d_min;
  tm.tm_hour = dt.d_hour;
  tm.tm_mday = dt.d_day;
  tm.tm_mon = dt.d_month - 1;
  tm.tm_year = dt.d_year;
  tm.tm_wday = dt.d_wday;
  tm.tm_yday = dt.d_days - 1;
  tm.tm_isdst = 0; /* cannot resolve this */

  return tm;
}

/* BTRON */
time_t time(time_t *t)
{
  W err;
  STIME tm;
  time_t ret;

  err = get_tim(&tm, NULL);
  if (err < 0) {
    ret = (time_t)-1;
    goto label_ret;
  }

  /* adds the unix time for 1985-01-01 00:00:00 GMT */
  tm += 473385600;
  ret = (time_t)tm;

label_ret:
  if (t != NULL) {
    t = ret;
  }
  return ret;
}

